@@ -10,6 +10,7 @@ from lsimodel import LsiModel
 from tfidfmodel import TfidfModel
 from rpmodel import RpModel
 from logentropy_model import LogEntropyModel
+from word2vec import Word2Vec
 
 from gensim import interfaces, utils
 
@@ -46,5 +47,5 @@ class VocabTransform(interfaces.TransformationABC):
         if is_corpus:
             return self._apply(bow)
 
-        return [(self.old2new[oldid], weight) for oldid, weight in bow if oldid in self.old2new]
+        return sorted((self.old2new[oldid], weight) for oldid, weight in bow if oldid in self.old2new)
 #endclass VocabTransform
