@@ -496,15 +496,31 @@ else:
                 yield chunk
 
 
-def smart_open(fname, mode):
+def make_closing(base, **attrs):
+    """
+    Add support for `with Base(attrs) as fout:` to the base class if it's missing.
+    The base class' `close()` method will be called on context exit, to always close the file properly.
+
+    This is needed for gzip.GzipFile, bz2.BZ2File etc in older Pythons (<=2.6), which otherwise
+    raise "AttributeError: GzipFile instance has no attribute '__exit__'".
+
+    """
+    if not hasattr(base, '__enter__'):
+        attrs['__enter__'] = lambda self: self
+    if not hasattr(base, '__exit__'):
+        attrs['__exit__'] = lambda self, type, value, traceback: self.close()
+    return type('Closing' + base.__name__, (base, object), attrs)
+
+
+def smart_open(fname, mode='r'):
     from os import path
     _, ext = path.splitext(fname)
     if ext == '.bz2':
         from bz2 import BZ2File
-        return BZ2File(fname, mode)
+        return make_closing(BZ2File)(fname, mode)
     if ext == '.gz':
         from gzip import GzipFile
-        return GzipFile(fname, mode)
+        return make_closing(GzipFile)(fname, mode)
     return open(fname, mode)
 
 
